<?php
/*
 * Phake - Mocking Framework
 *
 * Copyright (c) 2010-2012, Mike Lively <m@digitalsandwich.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *  *  Neither the name of Mike Lively nor the names of his
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * @category   Testing
 * @package    Phake
 * @author     Mike Lively <m@digitalsandwich.com>
 * @copyright  2010 Mike Lively <m@digitalsandwich.com>
 * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @link       http://www.digitalsandwich.com/
 */

require_once('Phake/Matchers/IMethodMatcher.php');

/**
 * Matches any abstract methods.
 *
 * This matcher is always registered in each mock's stub mapper to ensure that for partial mocks, abstract methods are
 * set up to return null;
 *
 * @author Mike Lively <m@digitalsandwich.com>
 */
class Phake_Matchers_AbstractMethodMatcher implements Phake_Matchers_IMethodMatcher
{
	/**
	 * @var \ReflectionClass
	 */
	private $mockedClass;

	/**
	 * @param ReflectionClass $mockedClass The class that is being mocked
	 */
	public function __construct(ReflectionClass $mockedClass)
	{
		$this->mockedClass = $mockedClass;
	}

	/**
	 * Determines if the given method and arguments match the configured method and argument matchers
	 * in this object. Returns true on success, false otherwise.
	 *
	 * @param string $method
	 * @param array $args
	 * @return boolean
	 */
	public function matches($method, array &$args)
	{
		$matches = FALSE;
		if ($this->mockedClass->hasMethod($method))
		{
			$reflMethod = $this->mockedClass->getMethod($method);
			$matches = $reflMethod->isAbstract();
		}

		return $matches;
	}
}
